import * as React from 'react';
import classNames from 'classnames';
import CodeMirror from 'react-codemirror';
import { setIn, removeIn } from 'immutable';
import { SortableContainer, SortableElement, SortableHandle, arrayMove } from 'react-sortable-hoc';
import dayjs from 'dayjs';
import TimePicker from './time-picker';
import DatePicker from './date-picker';
import { MediaItem } from '../types/media';
import { random, setTooltips } from '../lib/utility';
import MediaInsert from './media-insert';
import MediaUpdate from './media-update';
import Toggle from 'react-toggle';
import 'react-toggle/style.css';
import 'codemirror/lib/codemirror.css';
import 'codemirror/mode/javascript/javascript';
import 'codemirror/mode/xml/xml';
import { borderRadius } from 'react-select/lib/theme';

const DragHandle = SortableHandle(() => <span className="acms-admin-icon-sort acms-admin-banner-edit-item-handle" />);

interface BannerItem {
  id: string,
  media_banner_type: 'media' | 'source',
  media_banner_attr1?: string,
  media_banner_attr2?: string,
  media_banner_source: '',
  media_banner_status: 'true' | 'false',
  media_banner_preview: string,
  media_banner_datestart: string,
  media_banner_timestart: string,
  media_banner_dateend: string,
  media_banner_timeend: string,
  media_banner_mid: string,
  media_banner_alt: string,
  media_banner_landscape?: boolean
}

interface BannerEditState {
  isOpenAll: boolean,
  insertModalOpened: boolean,
  updateModalOpened: boolean,
  targetItem: BannerItem,
  modalType?: 'select' | 'upload',
  items: BannerItem[],
  changed: boolean
}

export default class BannerEdit extends React.Component<{}, BannerEditState> {

  page: HTMLElement;
  SortableList = this.renderList();
  SortableItem = this.renderItem();
  SortableUnitBox = this.renderUnitBox();

  constructor(props) {
    super(props);
    const items = props.items ? this.convertToBannerItems(props.items) : [];
    this.state = {
      isOpenAll: false,
      insertModalOpened: false,
      updateModalOpened: false,
      targetItem: null,
      modalType: 'upload',
      items: [{}, ...items],
      changed: false
    };
  }

  componentDidMount() {
    const {items} = this.state;
    if (items.length < 2) {
      this.addItem('image');
    }
  }

  componentDidUpdate() {
    if (this.state.changed) {
      $('.js-config-not-saved').addClass('active');
    }
    setTooltips(this.page);
  }

  defineDefaultItem(): BannerItem {
    return {
      id: random(10),
      media_banner_source: "",
      media_banner_datestart: dayjs().format('YYYY-MM-DD'),
      media_banner_timestart: "00:00:00",
      media_banner_dateend: "9999-12-31",
      media_banner_timeend: "23:59:59",
      media_banner_landscape: true,
      media_banner_attr1: "",
      media_banner_attr2: "",
      media_banner_alt: "",
      media_banner_status: "true",
      media_banner_type: "media",
      media_banner_preview: "",
      media_banner_mid: ""
    };
  }


  convertToBannerItems(items) : BannerItem[] {
    return items.map((item) => ({
      ...item,
      id: random(10)
    }));
  }

  addItem(type: 'image' | 'source') {
    const { items } = this.state;
    const defaultItem = this.defineDefaultItem();
    const item = setIn(defaultItem, ['media_banner_type'], type);
    const findIndex = items.findIndex((item) => !item.id);
    const newItems = [...items.slice(0, findIndex + 1), item, ...items.slice(findIndex + 1)];

    this.setState({
      items: [...newItems],
      changed: true
    });
  }

  removeItem(id: string) {
    const { items } = this.state;
    const index = items.findIndex((item) => {
      if (item.id === id) {
        return true;
      }
      return false;
    });
    this.setState({
      items: removeIn(items, [index]),
      changed: true
    });
  }

  updateItem(id: string, key: keyof BannerItem, value) {
    const { items } = this.state;
    const item = items.find((item) => item.id === id);
    const newItem = setIn(item, [key], value);
    const index = items.findIndex((item) => item.id === id);
    this.setState({
      items: setIn(items, [index], newItem),
      changed: true
    });
  }

  openInsertModal(item) {
    this.setState({
      targetItem: item,
      modalType: 'select',
      insertModalOpened: true
    });
  }

  openUploadModal(item) {
    this.setState({
      targetItem: item,
      modalType: 'upload',
      insertModalOpened: true
    });
  }

  openUpdateModal(item) {
    this.setState({
      targetItem: item,
      updateModalOpened: true
    });
  }

  setNewItem(item?: MediaItem, banner: BannerItem = null, type: 'image' | 'source' = 'image'): BannerItem {
    const defaultItem = banner? banner : this.defineDefaultItem();
    let media_banner_landscape = true;

    if (item.media_size) {
      const [sizeX, sizeY] = item.media_size.split(' , ');
      if (sizeX < sizeY) {
        media_banner_landscape = false;
      }
    }

    return Object.assign({}, defaultItem, {
      media_banner_mid: item.media_id,
      media_banner_preview: item.media_thumbnail,
      media_banner_type: type,
      media_banner_link: item.media_link,
      media_banner_landscape
    });
  }

  onInsert(newItems: MediaItem[]) {
    const { targetItem, items } = this.state;
    const index = items.findIndex((item) => item.id === targetItem.id);
    const newItem = this.setNewItem(newItems[0], items[index]);

    let setinItems = setIn(items, [index], newItem);

    if (newItems.length > 1) {
      const firstItems = [...setinItems.slice(0, index + 1)];
      const lastItems = [...setinItems.slice(index + 1)];
      const middleItems = [...newItems.slice(1).map((item) => {
        return this.setNewItem(item, null);
      })];
      setinItems = [...firstItems, ...middleItems, ...lastItems];
    }

    this.setState({
      items: setinItems,
      insertModalOpened: false,
      changed: true
    });
  }

  onUpdateModalUpdate(item: MediaItem) {
    const { targetItem, items } = this.state;
    const index = items.findIndex((item) => item.id === targetItem.id);
    const banner = items[index];
    const newItem = this.setNewItem(item, banner);
    const newItems = setIn(items, [index], newItem);
    this.setState({
      items: newItems,
      updateModalOpened: false,
      changed: true
    });
  }

  onClose() {
    this.setState({
      insertModalOpened: false
    });
  }

  openAll() {
    const items = this.state.items;
    items.forEach((item) => {
      item.toggle = true;
    });
    this.setState({
      isOpenAll: true,
      items
    });
  }

  closeAll() {
    const items = this.state.items;
    items.forEach((item) => {
      item.toggle = false;
    });
    this.setState({
      isOpenAll: false
    });
  }

  onUpdateModalClose() {
    this.setState({
      updateModalOpened: false
    });
  }

  onSortEnd = ({oldIndex, newIndex}) => {
    const { items } = this.state;
    this.setState({
      items: arrayMove(items, oldIndex, newIndex),
      changed: true
    })
  }

  renderList() {
    return SortableContainer(({ children }) => {
      return(<div>{children}</div>);
    });
  }

  getValueByTime (time: Date) {
    dayjs(value).format('HH:ii:ss');
  }

  getTimeByValue (value) {
    return dayjs().set('hour', )
  }

  renderUnitBox() {
    return SortableElement(() => (
    <div className="ablogcms--column-add-btn acms-admin-banner-edit-item-wrap" style={{ margin: '0 0 15px 0' }}>
      <div style={{position: 'relative'}}>
        <DragHandle />
        <div style={{marginLeft: '40px'}}>
          <span className="acms-admin-inline-btn" style={{ fontSize: '14px', verticalAlign: 'middle', margin: '5px 15px 0 0' }}>追加</span>
          <div className="acms-admin-inline-btn">
            <button className="acms-admin-btn" onClick={this.addItem.bind(this, 'image')} type="button">メディア</button>
          </div>
          <div className="acms-admin-inline-btn">
            <button className="acms-admin-btn" style={{ marginRight: '5px' }} onClick={this.addItem.bind(this, 'source')} type="button">ソース</button>
          </div>
        </div>
      </div>
    </div>
    ));
  }

  renderItem() {
    return SortableElement(({ item }: { item: BannerItem }) => (
      <div className={classNames('acms-admin-form acms-admin-banner-edit-item-wrap', { 'acms-admin-nested-selected': item.selected, 'acms-admin-nested-private': !item.media_banner_status })} key={item}>
        <div className="acms-admin-banner-edit-item">
          <DragHandle />
          <button className="acms-admin-btn-admin acms-admin-btn-admin-danger acms-admin-banner-edit-remove-btn" onClick={this.removeItem.bind(this, item.id)}>削除</button>
          <div className="acms-admin-nested-item-header acms-admin-banner-edit clearfix">
            <div className="acms-admin-nested-item-inner">
              <div style={{ width: '100%' }}>
                {item.media_banner_type === 'image' &&
                  <>
                    <h3 className="acms-admin-banner-edit-item-title">メディア<i className="acms-admin-icon-tooltip js-acms-tooltip-hover acms-admin-margin-left-mini" data-acms-tooltip="表示される画像です。media_bannerモジュールではメディアに登録されている画像のみ選択できます。"></i></h3>
                    <div className="acms-admin-banner-edit-item-inner" style={{marginTop: '0'}}>
                      <div className="acms-admin-banner-edit-item-thumb-wrap" style={{position: 'relative'}}>
                        <div>
                          <div className={classNames("acms-admin-media-img-wrap", "acms-admin-banner-img-wrap", {
                            "acms-admin-media-img-landscape": item.media_banner_landscape })}>
                            {item.media_banner_landscape}
                            {item.media_banner_preview && <img alt={item.media_banner_alt} src={`${item.media_banner_preview}`} style={{ verticalAlign: 'middle', opacity: 1 }} />}
                            <div className="acms-admin-media-btn-group">
                              {item.media_banner_mid && <button type="button" className="acms-admin-media-edit-btn" onClick={() => {
                                this.openUpdateModal(item);
                              }}>編集</button>}
                            </div>
                          </div>
                        </div>
                      </div>
                      <div className="acms-admin-banner-edit-content">
                        <table className="acms-admin-banner-edit-table">
                          <tr>
                            <th className="acms-admin-table-nowrap">メディアID</th>
                            <td>
                              {item.media_banner_mid &&
                              <span className="acms-admin-label acms-admin-label-default" style={{marginRight: '5px'}}>{item.media_banner_mid}</span>}
                              <button type="button" className="acms-admin-btn" onClick={() => {
                                this.openInsertModal(item);
                              }}>メディアから選択</button>&nbsp;
                              <button type="button" className="acms-admin-btn" onClick={() => {
                                this.openUploadModal(item);
                              }}>アップロード</button>
                            </td>
                          </tr>
                          <tr>
                            <th className="acms-admin-table-nowrap">URL</th>
                            <td>
                              {item.media_banner_mid && <button type="button" className="acms-admin-btn" onClick={() => {
                                this.openUpdateModal(item);
                              }}>編集</button>}&nbsp;
                              <span style={{marginRight: '5px', display: 'inline-block'}}><a href={item.media_banner_link} target="_blank" rel="noopener noreferrer">{item.media_banner_link}</a></span>
                            </td>
                          </tr>
                          <tr>
                            <th className="acms-admin-table-nowrap">属性<i className="acms-admin-icon-tooltip js-acms-tooltip-hover acms-admin-margin-left-mini" data-acms-tooltip="任意の属性を設定できます。（変数名 attr1, attr2）"></i></th>
                            <td>
                              属性1 <input type="text" className="acms-admin-form-width-mini" id={`media_banner-item-attr1-${item.id}`} value={item.media_banner_attr1} onInput={(e) => { this.updateItem(item.id, 'media_banner_attr1', e.target.value); }} style={{marginRight: '5px'}} />
                              属性2 <input type="text" className="acms-admin-form-width-mini" id={`media_banner-item-attr2-${item.id}`} value={item.media_banner_attr2} onInput={(e) => { this.updateItem(item.id, 'media_banner_attr2', e.target.value); }} />
                            </td>
                          </tr>
                          <tr>
                            <th className="acms-admin-table-nowrap">alt<i className="acms-admin-icon-tooltip js-acms-tooltip-hover acms-admin-margin-left-mini" data-acms-tooltip="画像のaltを設定できます。（変数名 alt）"></i></th>
                            <td>
                              <input type="text" className="acms-admin-form-width-medium" id={`media_banner-item-alt-${item.id}`} value={item.media_banner_alt} onInput={(e) => { this.updateItem(item.id, 'media_banner_alt', e.target.value); }} />
                            </td>
                          </tr>
                          <tr>
                            <th className="acms-admin-table-nowrap">リンク<i className="acms-admin-icon-tooltip js-acms-tooltip-hover acms-admin-margin-left-mini" data-acms-tooltip="このバナーのリンクを新しいウィンドウで開くかのチェックです。チェックすると新しいウィンドウで開きます。"></i></th>
                            <td>
                            <div className="acms-admin-form-checkbox">
                              <input type="checkbox" checked={item.media_banner_target === 'true'}
                                id={`media_banner-item-target-${item.id}`}
                                value="true"
                                onChange={(e) => {
                                  if (item.media_banner_target === 'true') {
                                    this.updateItem(item.id, 'media_banner_target', '');
                                  } else {
                                    this.updateItem(item.id, 'media_banner_target', 'true');
                                  }
                                }}
                              />
                              <label htmlFor={`media_banner-item-target-${item.id}`}>
                                <i className="acms-admin-ico-checkbox"></i>新しいウィンドウで開く
                              </label>
                            </div>
                            </td>
                          </tr>

                        </table>
                      </div>
                    </div>
                  </>
                }
                {item.media_banner_type === 'source' &&
                  <>
                    <h3 className="acms-admin-banner-edit-item-title">ソース<i className="acms-admin-icon-tooltip js-acms-tooltip-hover acms-admin-margin-left-mini" data-acms-tooltip="HTMLを直接記述してバナーを掲載する場合、この欄にHTMLを入力してください。"></i></h3>
                    <CodeMirror
                      defaultValue={item.media_banner_source}
                      options={{ mode: 'xml' }}
                      style={{maxWidth: '820px', borderRadius: '3px'}}
                      onChange={(code) => {
                        this.updateItem(item.id, 'media_banner_source', code);
                      }}
                    />
                  </>
                }
                <div className="acms-admin-banner-edit-item-inner">
                  <div className="acms-admin-banner-edit-check-wrap">
                    <span className="acms-admin-banner-edit-open-label">公開設定<i className="acms-admin-icon-tooltip js-acms-tooltip-hover acms-admin-margin-left-mini" data-acms-tooltip="このバナーの公開日時を指定します。バナーの表示には日時の範囲内であっても、公開設定にチェックが入っている必要があります。"></i></span>
                    <Toggle
                      defaultChecked={item.media_banner_status}
                      onChange={(e) => {
                        e.stopPropagation();
                        this.updateItem(item.id, 'media_banner_status', e.target.checked);
                      }}
                      icons={{
                        checked: <span>公開</span>,
                        unchecked: <span>非公開</span>
                      }}
                    />
                  </div>
                  <div className="acms-admin-banner-edit-content">
                    <table className="acms-admin-banner-edit-table">
                      <tr>
                        <th className="acms-admin-padding-none">公開期限<i className="acms-admin-icon-tooltip js-acms-tooltip-hover acms-admin-margin-left-mini" data-acms-tooltip="このバナーの公開日時を指定します。バナーの表示には日時の範囲内であっても、公開設定にチェックが入っている必要があります。"></i></th>
                        <td className="acms-admin-padding-none">
                          <DatePicker style={{maxWidth: '95px'}} value={item.media_banner_datestart} onChange={(value) => {
                            this.updateItem(item.id, 'media_banner_datestart', value);
                          }}/>
                          &nbsp;
                          <TimePicker style={{maxWidth: '95px'}} value={item.media_banner_timestart} onChange={(value) => {
                            this.updateItem(item.id, 'media_banner_timestart', value);
                          }}/>
                          &nbsp;
                          〜
                          &nbsp;
                          <DatePicker style={{maxWidth: '95px'}} value={item.media_banner_dateend} onChange={(value) => {
                            this.updateItem(item.id, 'media_banner_dateend', value);
                          }} />
                          &nbsp;
                          <TimePicker style={{maxWidth: '95px'}} value={item.media_banner_timeend} onChange={(value) => {
                            this.updateItem(item.id, 'media_banner_timeend', value);
                          }}/>
                        </td>
                      </tr>
                    </table>
                  </div>
                </div>
              </div>
            </div>
          </div>
        </div>
        <textarea style={{display: 'none'}} name="media_banner_source[]" value={item.media_banner_source ? item.media_banner_source : ''}></textarea>
        <input type="hidden" name="media_banner_datestart[]" value={item.media_banner_datestart ? item.media_banner_datestart : ''} />
        <input type="hidden" name="media_banner_timestart[]" value={item.media_banner_timestart ? item.media_banner_timestart : ''} />
        <input type="hidden" name="media_banner_dateend[]" value={item.media_banner_dateend ? item.media_banner_dateend : ''} />
        <input type="hidden" name="media_banner_timeend[]" value={item.media_banner_timeend ? item.media_banner_timeend : ''} />
        <input type="hidden" name="media_banner_mid[]" value={item.media_banner_mid ? item.media_banner_mid : ''} />
        <input type="hidden" name="media_banner_attr1[]" value={item.media_banner_attr1 ? item.media_banner_attr1 : ''} />
        <input type="hidden" name="media_banner_attr2[]" value={item.media_banner_attr2 ? item.media_banner_attr2 : ''} />
        <input type="hidden" name="media_banner_status[]" value={item.media_banner_status ? item.media_banner_status : ''} />
        <input type="hidden" name="media_banner_target[]" value={item.media_banner_target ? item.media_banner_target : ''} />
        <input type="hidden" name="media_banner_type[]" value={item.media_banner_type} />
        <input type="hidden" name="media_banner_alt[]" value={item.media_banner_alt} />
      </div>
    ))
  }

  render() {
    const { insertModalOpened, updateModalOpened, targetItem, modalType, items } = this.state;
    const { SortableList, SortableItem, SortableUnitBox } = this;
    return (
      <div ref={(page) => this.page = page}>
        <SortableList useDragHandle={true} axis="y" onSortEnd={this.onSortEnd}>
          {items.map((item, index) => {
            if (item.id) {
              return (<SortableItem item={item} index={index} key={item.id} />)
            } else {
              return (<SortableUnitBox index={index} />)
            }
          })}
        </SortableList>
        <input type="hidden" name="@media_banner[]" value="media_banner_source" />
        <input type="hidden" name="config[]" value="media_banner_source" />
        <input type="hidden" name="@media_banner[]" value="media_banner_datestart" />
        <input type="hidden" name="config[]" value="media_banner_datestart" />
        <input type="hidden" name="@media_banner[]" value="media_banner_timestart" />
        <input type="hidden" name="config[]" value="media_banner_timestart" />
        <input type="hidden" name="@media_banner[]" value="media_banner_dateend" />
        <input type="hidden" name="config[]" value="media_banner_dateend" />
        <input type="hidden" name="@media_banner[]" value="media_banner_timeend" />
        <input type="hidden" name="config[]" value="media_banner_timeend" />
        <input type="hidden" name="@media_banner[]" value="media_banner_mid" />
        <input type="hidden" name="config[]" value="media_banner_mid" />
        <input type="hidden" name="@media_banner[]" value="media_banner_attr1" />
        <input type="hidden" name="config[]" value="media_banner_attr1" />
        <input type="hidden" name="@media_banner[]" value="media_banner_attr2" />
        <input type="hidden" name="config[]" value="media_banner_attr2" />
        <input type="hidden" name="@media_banner[]" value="media_banner_status" />
        <input type="hidden" name="config[]" value="media_banner_status" />
        <input type="hidden" name="@media_banner[]" value="media_banner_target" />
        <input type="hidden" name="config[]" value="media_banner_target" />
        <input type="hidden" name="@media_banner[]" value="media_banner_type" />
        <input type="hidden" name="config[]" value="media_banner_type" />
        <input type="hidden" name="config[]" value="@media_banner" />
        <input type="hidden" name="config[]" value="media_banner_alt" />
        <input type="hidden" name="@media_banner[]" value="media_banner_alt" />
        {insertModalOpened && <MediaInsert onInsert={this.onInsert.bind(this)} onClose={this.onClose.bind(this)} tab={modalType} filetype="image" />
        }
        {updateModalOpened && <MediaUpdate
          mid={targetItem.media_banner_mid}
          onClose={this.onUpdateModalClose.bind(this)}
          onUpdate={this.onUpdateModalUpdate.bind(this)}
        />}
      </div>
    );
  }
}
